# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Application constants. All defaults can be changed when initializing a client
via the MicrosoftAgentsM365CopilotClient or HttpClientFactory
"""
DEFAULT_REQUEST_TIMEOUT = 100
DEFAULT_CONNECTION_TIMEOUT = 30
# The SDK version
# x-release-please-start-version
SDK_VERSION = '1.0.0-preview.1'
# x-release-please-end
MS_DEFAULT_SCOPE = 'https://graph.microsoft.com/.default'
